<?php

if (!defined("SMF"))
	die("");

require_once(dirname( __FILE__ ) . "/ChatweeV2_SDK/Chatwee.php");

function set_chatwee_plugin_options($return_config = false) {
	global $txt, $scripturl, $context;

	$config_vars = Array(
		Array("check", "enable_chatwee"),
		Array("large_text", "chatwee_code"),
		Array("check", "enable_sso"),
		Array("text", "chatwee_id", 33),
		Array("text", "chatwee_api_key", 33),
		Array("check", "global_moderators_as_admin"),
		Array("desc", "export_configuration")
	);

	if ($return_config) {
		return $config_vars;
	}

	if (isset($_GET["save"])) {
		checkSession();
		saveDBSettings($config_vars);
		save_config();
		redirectexit("action=admin;area=modsettings;sa=chatwee");
	}

	$context["post_url"] = $scripturl . "?action=admin;area=modsettings;save;sa=chatwee";
	$context["settings_title"] = $txt["chatwee"];

	prepareDBSettingContext($config_vars);
}

function register_chatwee_settings($regOptions) {
	$username = $regOptions["username"];
	remote_login($username);
};

function modify_modifications_settings(&$config_vars) {
	$config_vars["chatwee"] = "set_chatwee_plugin_options";
}

function chatwee_code_load_theme() {
	global $context, $modSettings;

	synchronize_sso_user();

	if($modSettings["chatwee_code"] && $modSettings["enable_chatwee"]) {
		if (!empty($modSettings["chatwee_code"]) && $modSettings["enable_chatwee"] == 1) {
			$context["insert_after_template"] .= $modSettings["chatwee_code"];
		}
	}
}

function set_sso_credentials() {
	global $modSettings;

	ChatweeV2_Configuration::setChatId($modSettings["chatwee_id"]);
	ChatweeV2_Configuration::setClientKey($modSettings["chatwee_api_key"]);
	ChatweeV2_Configuration::setCustomUserAgent("Chatwee SMF Plugin 2.2");
}

function synchronize_sso_user() {
	global $context, $modSettings;

	if(!empty($_SERVER["HTTP_X_REQUESTED_WITH"]) && strtolower($_SERVER["HTTP_X_REQUESTED_WITH"]) === "xmlhttprequest") {
		return false;
	}

	if($modSettings["enable_chatwee"] == 1 && $modSettings["enable_sso"] == 1) {

		$user = $context["user"];

		if(!$user["is_logged"]) {
			return false;
		}

		set_sso_credentials();

		try {
			if(ChatweeV2_Session::isSessionSet() === false) {
				sso_login($user["username"]);
			} else {
				$sessionId = ChatweeV2_Session::getSessionId();

				$isSessionValid = ChatweeV2_SsoUser::validateSession(Array(
					"sessionId" => $sessionId
				));

				if($isSessionValid === false) {
					sso_login($user["username"]);
				}
			}
		} catch(Exception $exception) {
			log_error("Error while reinitializng SSO session: " . $exception->getMessage());
		}
	}
}

function remote_login($username) {
	global $modSettings;

	if ($modSettings["enable_chatwee"] == 1 && $modSettings["enable_sso"] == 1) {
		sso_login($username);
	}
}

function sso_login($username) {
	global $modSettings, $smcFunc, $db_prefix;

	if($modSettings["chatwee_id"] && $modSettings["chatwee_api_key"]) {

		set_sso_credentials();

		$user_chatwee_id = fetch_user_chatwee_id($username);

		if(!$user_chatwee_id) {
			$user_chatwee_id = sso_register($username);
		}

		try	{
			ChatweeV2_SsoUser::edit(Array(
				"userId" => $user_chatwee_id,
				"login" => fetch_user_real_name($username) ? fetch_user_real_name($username) : $username,
				"avatar" => fetch_user_avatar($username),
				"isAdmin" => isSet($modSettings["global_moderators_as_admin"]) && $modSettings["global_moderators_as_admin"] == 1 ? is_global_moderator_or_admin($username) : false
			));
		} catch(Exception $exception) {
			if(strpos($exception->getMessage(), "User does not exist") !== false) {
				$user_chatwee_id = sso_register($username);
			} else {
				log_error("Error while executing sso_login: " . $exception->getMessage());
			}
		}

		try {
			ChatweeV2_SsoManager::loginUser(Array(
				"userId" => $user_chatwee_id
			));
		} catch(Exception $exception) {
			log_error("Error while executing sso_login: " . $exception->getMessage());
		}
	}
}

function fetch_user_chatwee_id($username) {
	global $smcFunc, $db_prefix;

	$query_parameters = Array("username" => $username);
	$query = $smcFunc["db_query"]("", "SELECT chatwee_id FROM {db_prefix}users_chatwee_id WHERE username = {string:username} LIMIT 1", $query_parameters);
	$row = $smcFunc["db_fetch_row"] ($query);
	$result = $row[0];

	return $result ? $result : null;
}

function sso_register($username) {
	global $smcFunc, $db_prefix;

	try {
		$userId = ChatweeV2_SsoManager::registerUser(Array(
			"login" => fetch_user_real_name($username) ? fetch_user_real_name($username) : $username,
			"avatar" => fetch_user_avatar($username),
			"isAdmin" => isSet($modSettings["global_moderators_as_admin"]) && $modSettings["global_moderators_as_admin"] == 1 ? is_global_moderator_or_admin($username) : false
		));

		$query_parameters = Array("username" => $username);
		$smcFunc['db_query']("", "DELETE FROM {db_prefix}users_chatwee_id WHERE username = {string:username}", $query_parameters);

		$values_to_change = Array("username" => "string", "chatwee_id" => "string");
		$data_to_insert = Array($username, $userId);
		$smcFunc["db_insert"] ("insert", "{db_prefix}users_chatwee_id", $values_to_change, $data_to_insert);
		return $userId;

	} catch(Exception $exception) {
		log_error("Error while executing sso_register: " . $exception->getMessage());
		return null;
	}
}

function remote_logout($username) {
	global $modSettings;

	if ($modSettings["enable_chatwee"] == 1 && $modSettings["enable_sso"] == 1) {
		sso_logout($username);
	}
}

function sso_logout($username) {
	global $modSettings;

	if ($modSettings["chatwee_id"] && $modSettings["chatwee_api_key"]) {
		try {
	    	ChatweeV2_SsoManager::logoutUser();
	  	} catch(Exception $exception) {
			log_error("Error while executing sso_logout: " . $exception->getMessage());
		}
	}
}

function fetch_user_avatar($username) {
	global $smcFunc, $db_prefix;

	$query_parameters = Array("member_name" => $username);

	$query = $smcFunc["db_query"]("", "SELECT avatar FROM {db_prefix}members WHERE member_name = {string:member_name}", $query_parameters);
	$row = $smcFunc["db_fetch_row"]($query);
	$avatar = $row[0];

	return $avatar;
}

function is_global_moderator_or_admin($username) {
	global $smcFunc, $db_prefix;

	$query_parameters = Array("member_name" => $username);
	$query = $smcFunc["db_query"]("", "SELECT id_group FROM {db_prefix}members WHERE member_name = {string:member_name}", $query_parameters);
	$row = $smcFunc["db_fetch_row"] ($query);
	$result = $row[0];

	return $result > 0 ? ($result <= 2 ? true : false) : false;
}

function fetch_user_real_name($username) {
	global $smcFunc, $db_prefix;

	$query_parameters = Array("member_name" => $username);

	$query = $smcFunc["db_query"]("", "SELECT real_name FROM {db_prefix}members WHERE member_name = {string:member_name}", $query_parameters);
	$row = $smcFunc["db_fetch_row"] ($query);
	$real_name = $row[0];

	return $real_name;
}

function save_config() {
	global $modSettings;

	$keys = Array(
		"enable_chatwee",
		"chatwee_code",
		"enable_sso",
		"chatwee_id",
		"chatwee_api_key",
		"global_moderators_as_admin"
		);

	$settings = Array();
	foreach($keys as $key) {
		$settings[$key] = $modSettings[$key];
	}

	$file = "chatwee_config.json";
	$current = file_get_contents($file);
	$current = json_encode($settings);
	file_put_contents($file, $current);
}

?>